//Added numextras
//Added sector[].ceilingpal
//Added sector[].floorpal
//Added sector[].visibility
//Split sector[].tag to sector[].lotag and sector[].hitag
//Added sector[].extra
//Expanded wall[].cstat to a short
//Split wall[].tag to wall[].lotag and wall[].hitag
//Added wall[].extra
//Split sprite[].tag to sprite[].lotag and sprite[].hitag
//Got rid of sprite[].extra (the void * mess)
//Added sprite[].extra

#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>

#define NEWMAPVERSION 5

#define MAXSECTORS 1024
#define MAXWALLS 4096
#define MAXSPRITES 4096
#define MAXEXTRAS 4096

#define MAXSTATUS 1024
#define MAXTILES 4096

#ifdef __GNUC__
#define BPACK __attribute__ ((packed))
#else
#define BPACK
#endif

#ifdef _MSC_VER
#pragma pack(1)
#endif

#ifdef __WATCOMC__
#pragma pack(push,1);
#endif

typedef struct BPACK
{
	short unsigned int wallptr, wallnum;
	char ceilingstat, ceilingxpanning, ceilingypanning;
	signed char ceilingshade;
	int ceilingz;
	short ceilingpicnum, ceilingheinum;
	char floorstat, floorxpanning, floorypanning;
	signed char floorshade;
	int floorz;
	short floorpicnum, floorheinum;
	int tag;
} sectortype;

typedef struct BPACK
{
	int x, y;
	short point2;
	char cstat;
	signed char shade;
	unsigned char xrepeat, yrepeat, xpanning, ypanning;
	short picnum, overpicnum;
	short nextsector1, nextwall1;
	short nextsector2, nextwall2;
	int tag;
} walltype;

typedef struct BPACK
{
	int x, y, z;
	char cstat;
	signed char shade;
	unsigned char xrepeat, yrepeat;
	short picnum, ang, xvel, yvel, zvel, owner;
	short sectnum, statnum;
	int tag;
	char *extra;
} spritetype;

typedef struct BPACK
{
	short unsigned int wallptr, wallnum;
	short ceilingpicnum, floorpicnum;
	short ceilingheinum, floorheinum;
	int ceilingz, floorz;
	signed char ceilingshade, floorshade;
	char ceilingxpanning, floorxpanning;
	char ceilingypanning, floorypanning;
	char ceilingstat, floorstat;
	char ceilingpal, floorpal;
	char visibility;
	short lotag, hitag;
	short extra;
} newsectortype;

typedef struct BPACK
{
	int x, y;
	short point2;
	short picnum, overpicnum;
	signed char shade;
	short cstat;
	unsigned char xrepeat, yrepeat, xpanning, ypanning;
	short nextsector1, nextwall1;
	short nextsector2, nextwall2;
	short lotag, hitag;
	short extra;
} newwalltype;

typedef struct BPACK
{
	int x, y, z;
	char cstat;
	signed char shade;
	unsigned char xrepeat, yrepeat;
	short picnum, ang, xvel, yvel, zvel, owner;
	short sectnum, statnum;
	short lotag, hitag;
	short extra;
} newspritetype;

typedef struct BPACK
{
	short tag[4];
	short point2;
} newextratype;

#ifdef _MSC_VER
#pragma pack()
#endif

#ifdef __WATCOMC__
#pragma pack(pop)
#endif

//**************************************************************************

static sectortype sector[MAXSECTORS];
static walltype wall[MAXWALLS];
static spritetype sprite[MAXSPRITES];

static newsectortype newsector[MAXSECTORS];
static newwalltype newwall[MAXWALLS];
static newspritetype newsprite[MAXSPRITES];
static newextratype newextra[MAXEXTRAS];

static int posx, posy, posz, mapversion;
static short int ang, cursectnum;
static short numsectors, numwalls, numsprites, numextras;

int loadoldboard (char *filename)
{
	short int fil, i;

	if ((fil = open(filename,O_BINARY|O_RDWR,S_IREAD)) == -1)
		return(-1);

	read(fil,&mapversion,4);
	if (mapversion != 4L) return(-2);
	read(fil,&posx,4);
	read(fil,&posy,4);
	read(fil,&posz,4);
	read(fil,&ang,2);
	read(fil,&cursectnum,2);
	read(fil,&numsectors,2);
	read(fil,&numwalls,2);
	read(fil,&numsprites,2);
	for(i=0;i<numsectors;i++) read(fil,&sector[i],sizeof(sectortype));
	for(i=0;i<numwalls;i++) read(fil,&wall[i],sizeof(walltype));
	for(i=0;i<numsprites;i++) read(fil,&sprite[i],sizeof(spritetype));
	close(fil);

	return(0);
}

int savenewboard (char *filename)
{
	short int fil, i;

	if ((fil = open(filename,O_BINARY|O_TRUNC|O_CREAT|O_WRONLY,S_IWRITE)) == -1)
		return(-1);

		 //Version control
	write(fil,&mapversion,4);

		 //Starting position
	write(fil,&posx,4);
	write(fil,&posy,4);
	write(fil,&posz,4);
	write(fil,&ang,2);
	write(fil,&cursectnum,2);

		//Sectors
	write(fil,&numsectors,2);
	for(i=0;i<numsectors;i++)
		write(fil,&newsector[i],sizeof(newsectortype));

		//Walls
	write(fil,&numwalls,2);
	for(i=0;i<numwalls;i++)
		write(fil,&newwall[i],sizeof(newwalltype));

		//Sprites
	write(fil,&numsprites,2);
	for(i=0;i<numsprites;i++)
		write(fil,&newsprite[i],sizeof(newspritetype));

		//Extras
	write(fil,&numextras,2);
	for(i=0;i<numextras;i++)
		write(fil,&newextra[i],sizeof(newextratype));

	close(fil);

	return(0);
}

void convmap (char *ofilename)
{
	int i, j;
	char filename[260];

	strcpy(filename,ofilename);
	if (strchr(filename,'.') == 0) strcat(filename,".map");
	if ((i = loadoldboard(filename)) < 0)
	{
		if (i == -1)
		{
			printf("%s not found.\n",filename);
		}
		if (i == -2)
		{
			if (mapversion == NEWMAPVERSION)
				printf("%s already map version %ld.\n",filename,NEWMAPVERSION);
			else
				printf("CANNOT CONVERT %s!\n",filename);
		}
		return;
	}

	for(i=0;i<numsectors;i++)
	{
		newsector[i].wallptr = sector[i].wallptr;
		newsector[i].wallnum = sector[i].wallnum;
		newsector[i].ceilingpicnum = sector[i].ceilingpicnum;
		newsector[i].floorpicnum = sector[i].floorpicnum;
		newsector[i].ceilingheinum = sector[i].ceilingheinum;
		newsector[i].floorheinum = sector[i].floorheinum;
		newsector[i].ceilingz = sector[i].ceilingz;
		newsector[i].floorz = sector[i].floorz;
		newsector[i].ceilingshade = sector[i].ceilingshade;
		newsector[i].floorshade = sector[i].floorshade;
		newsector[i].ceilingxpanning = sector[i].ceilingxpanning;
		newsector[i].floorxpanning = sector[i].floorxpanning;
		newsector[i].ceilingypanning = sector[i].ceilingypanning;
		newsector[i].floorypanning = sector[i].floorypanning;
		newsector[i].ceilingstat = sector[i].ceilingstat;
		newsector[i].floorstat = sector[i].floorstat;
		newsector[i].ceilingpal = 0;
		newsector[i].floorpal = 0;
		newsector[i].visibility = 0;
		newsector[i].lotag = (sector[i].tag&65535);
		newsector[i].hitag = (sector[i].tag>>16);
		newsector[i].extra = -1;
	}
	for(i=0;i<numwalls;i++)
	{
		newwall[i].x = wall[i].x;
		newwall[i].y = wall[i].y;
		newwall[i].point2 = wall[i].point2;
		newwall[i].picnum = wall[i].picnum;
		newwall[i].overpicnum = wall[i].overpicnum;
		newwall[i].shade = wall[i].shade;
		newwall[i].cstat = wall[i].cstat;
		newwall[i].xrepeat = wall[i].xrepeat;
		newwall[i].yrepeat = wall[i].yrepeat;
		newwall[i].xpanning = wall[i].xpanning;
		newwall[i].ypanning = wall[i].ypanning;
		newwall[i].nextsector1 = wall[i].nextsector1;
		newwall[i].nextwall1 = wall[i].nextwall1;
		newwall[i].nextsector2 = wall[i].nextsector2;
		newwall[i].nextwall2 = wall[i].nextwall2;
		newwall[i].lotag = (wall[i].tag&65535);
		newwall[i].hitag = (wall[i].tag>>16);
		newwall[i].extra = -1;
	}
	for(i=0;i<numsprites;i++)
	{
		newsprite[i].x = sprite[i].x;
		newsprite[i].y = sprite[i].y;
		newsprite[i].z = sprite[i].z;
		newsprite[i].cstat = sprite[i].cstat;
		newsprite[i].shade = sprite[i].shade;
		newsprite[i].xrepeat = sprite[i].xrepeat;
		newsprite[i].yrepeat = sprite[i].yrepeat;
		newsprite[i].picnum = sprite[i].picnum;
		newsprite[i].ang = sprite[i].ang;
		newsprite[i].xvel = sprite[i].xvel;
		newsprite[i].yvel = sprite[i].yvel;
		newsprite[i].zvel = sprite[i].zvel;
		newsprite[i].owner = sprite[i].owner;
		newsprite[i].sectnum = sprite[i].sectnum;
		newsprite[i].statnum = sprite[i].statnum;
		newsprite[i].lotag = (sprite[i].tag&65535);
		newsprite[i].hitag = (sprite[i].tag>>16);
		newsprite[i].extra = -1;
	}

	mapversion = 5L;
	numextras = 0;
	if (savenewboard(filename) == 0)
		printf("%s successfully converted.\n",filename);
	else
		printf("%s NOT converted for some stupid reason.\n",filename);
}

int main (int argc, char **argv)
{
	if (argc != 2)
	{
		printf("CONVMAP%ld [filespec]                                            by Ken Silverman\n",NEWMAPVERSION);
		printf("This CONVMAP%ld.EXE converts map version %ld to map version %ld\n",NEWMAPVERSION,NEWMAPVERSION-1,NEWMAPVERSION);
		exit(0);
	}

	printf("Converting map version %ld to map version %ld\n",NEWMAPVERSION-1,NEWMAPVERSION);
	convmap(argv[1]);
	return(0);
}
